import { Component, OnInit, AfterContentInit, AfterViewInit, ViewChild, Inject, ChangeDetectorRef } from '@angular/core';
import { MapOptions, Point, PolylineOptions, BPolyline, ControlAnchor } from 'angular2-baidu-map';
import { DynamicScriptLoaderService } from '../dynamic-script.service';


import { theme } from './theme';
import { ModalService } from '../services';
import { DefaultService, Project, Course, Scetion } from '../api';
import { HttpClient, HttpParams } from '@angular/common/http';
import { timer } from 'rxjs';

import { timeInterval, pluck, take } from 'rxjs/operators';
import { ActivatedRoute } from '@angular/router';

declare var map: any;
declare var BMap: any;
declare var BMapLib: any;
const enum CordType {
  PROJ,
  COURSE,
  SECTION
}
const enum markType {
  HIGHWAY = 1,
  NORMAL
}
export interface DialogData {
  animal: string;
  name: string;
}
@Component({
  selector: 'app-main',
  templateUrl: './main.component.html',
  styleUrls: ['./main.component.less']
})
export class MainComponent implements OnInit {

  courses = [];
  highShow = false;
  normalShow = false;
  public opts: MapOptions
  public points: Array<Point> = [];
  public polylineOptions: PolylineOptions;

  public projects = [
  ]
  polys = [];
  proj: Project = { delFlag: 0 };
  course: Course = { delFlag: 0 };
  section: Scetion = { delFlag: 0 };
  type = 1;
  _this = this;
  currentCouseId: any;
  staticHighWay: any = {};
  staticNormal: any = {};
  normalProjects: any = [];
  debug = false;
  toPoints(points: Array<Point>) {
    if (!points) {
      return []
    }
    return points.map(p => {
      return new window.BMap.Point(p.lng, p.lat)
    })
  }

  addPoly(proj: Project) {
    var points = JSON.parse(proj.coordinate).map(c => ({ lng: c[0], lat: c[1] }));
    var newPolyLine = new window.BMap.Polyline(this.toPoints(points), { strokeColor: proj.lineStyle, strokeWeight: proj.lineWidth });
    map.addOverlay(newPolyLine);

  }
  ngOnInit(): void {
    this.route.queryParamMap.subscribe(queryParams => {
      this.debug =Boolean( queryParams.get("debug"))
    })
    this.http.get<any>(this.s.basePath + `/project/total`).subscribe(res => {
      this.staticHighWay = res.data[0];
    });
    this.http.get<any>(this.s.basePath + `/project/total2`).subscribe(res => {
      this.staticNormal = res.data[0];
    });

    this.http.get<any>(this.s.basePath + "/course/list").subscribe(res => {
      this.courses = res.data;
    });
  }
  toHighWay() {
  
    this.highShow = !this.highShow;
    
    if(this.highShow){
      map.clearOverlays();
  this.http.get<any>(this.s.basePath + `/project/list/${markType.HIGHWAY}`).subscribe(res => {
    this.projects = res.data;
    var allPoints = [];
    
    res.data.forEach(poly => {
      this.addPoly(poly);
      allPoints = allPoints.concat(JSON.parse(poly.coordinate));
    });
    map.setViewport(allPoints.map(p => ({ lng: p[0], lat: p[1] })));
  });
}

  }
  toNormal() {
    map.clearOverlays();
    this.normalShow = !this.normalShow;


    this.http.get<any>(this.s.basePath + `/project/list/${markType.NORMAL}`).subscribe(res => {
      this.normalProjects = res.data;

    });

  }
  openDialog(id: string) {
    this.modalService.open(id);
  }


  constructor(private route:ActivatedRoute, private cd: ChangeDetectorRef, private http: HttpClient, private dynamicScriptLoader: DynamicScriptLoaderService, private modalService: ModalService, private s: DefaultService) {
    //定位山西省地图
    this.opts = {
      centerAndZoom: {
        lat: 37.78663390039693,
        lng: 112.43176242674386,
        zoom: 15
      },

    }




    this.polylineOptions = {
      strokeColor: 'yellow',
      strokeWeight: 10
    }

  }
  corToPoint(json:string){
   return JSON.parse(json).map(r=>({lng:r[0],lat:r[1]}));
  }
  highLight(i:number,proj:Project) {
    var overlays = map.getOverlays();
    timer(0, 200)
      .pipe(timeInterval(), pluck('interval'), take(6)).subscribe(index => {
        if(overlays[i].isVisible()){
          overlays[i].hide();
        }else{
          overlays[i].show();
        }
      });

    map.setViewport(this.corToPoint(proj.coordinate))
  }
  

  addCourse(course: Course) {

    this.http.post<Course>(this.s.basePath + "/course/add", this.course).subscribe(res => {
      this.modalService.close('custom-modal-2');
    })
  }
  addSection(section: Scetion) {

    this.http.post<Scetion>(this.s.basePath + "/scetion/add", section).subscribe(res => {
      this.modalService.close('custom-modal-3');
    })
  }
  addProj(proj: Project) {
    this.http.post(this.s.basePath + "/project/add", proj).subscribe(res => {
      console.log(res)
    });
  }
  public polylineLoaded(polyline: BPolyline): void {
    (window as any).map = polyline.getMap();
    this.setBound();
    (window as any).map.setMapStyleV2({ styleJson: theme });
    this.dynamicScriptLoader.load('drawing', 'geouti').then(data => {
      map.enableScrollWheelZoom(true);
      // Script Loaded Successfully
      if(this.debug){

        var drawingManager = this.initDrawingTool();
        drawingManager.addEventListener("mousedown", e => {
          alert(e)
        });
      
      drawingManager.addEventListener('overlaycomplete', e => {
        var overlay = e.overlay;
        if (!e.overlay.getBounds) {
          return;
        }
        var bounds = e.overlay.getBounds();

        var ne = bounds.getNorthEast();
        var sw = bounds.getSouthWest();
        var mercatorProjection = map.getMapType().getProjection();
        var neMc = mercatorProjection.lngLatToPoint(ne);
        var swMc = mercatorProjection.lngLatToPoint(sw);
        var pathStr = "";
        var geojsonStr = {};
        var pathmcStr = "";
        var path = overlay.getPath();
        var coordinates = [];
        for (var i = 0; i < path.length; i++) {
          pathStr += path[i].lng + "," + path[i].lat + ",";
          var mc = mercatorProjection.lngLatToPoint(path[i]);
          pathmcStr += mc.x + "," + mc.y + ",";
          coordinates.push([path[i].lng, path[i].lat]);
        }
        pathmcStr = pathmcStr.substr(0, pathmcStr.length - 1);
        pathStr = pathStr.substr(0, pathStr.length - 1);
        if (overlay.toString() == '[object Polyline]') {
          geojsonStr = {
            "type": "LineString",
            "coordinates": coordinates
          };
        } else if (overlay.toString() == '[object Polygon]') {
          geojsonStr = {
            "type": "Polygon",
            "coordinates": [coordinates]
          };
        }

        if (this.type == CordType.PROJ) {
          this.proj.course = this.currentCouseId;
          this.proj.coordinate = JSON.stringify(coordinates);
          this.modalService.open('custom-modal-1');

        } else if (this.type == CordType.COURSE) {

          this.course.coordinate = JSON.stringify(coordinates);
          this.modalService.open('custom-modal-2');

        } else if (this.type == CordType.SECTION) {
          this.section.coordinate = JSON.stringify(coordinates);
          this.modalService.open('custom-modal-3');
        }
        document.getElementById('result').innerHTML = "<div><span>左下角,右上角(经纬度)：</span><button class='btn'>复制</button><p class='copyText'>" + sw.lng + "," + sw.lat + "," + ne.lng + "," + ne.lat + "</p></div>"
          + "<div><span>左下角,右上角(墨卡托坐标)：</span><button class='btn'>复制</button><p class='copyText'>" + swMc.x + "," + swMc.y + "," + neMc.x + "," + neMc.y + "</p></div>"
          + "<div><span>坐标集(经纬度)：</span><button class='btn'>复制</button><p class='copyText'>" + pathStr + "</p></div>"
          + "<div><span>坐标集(墨卡托坐标)：</span><button class='btn'>复制</button><p class='copyText'>" + pathmcStr + "</p></div>"
          + "<div><span>geojson：</span><button class='btn'>复制</button><p class='copyText' style='white-space:nowrap;'>" + JSON.stringify(geojsonStr) + "</p></div>";


      });
    }
    }).catch(error => console.log(error));
  
    console.log('polyline loaded', polyline)
  }

  public polylineClicked({ polyline }): void {
    console.log('polyline clicked', polyline)
  }

  overlaycomplete(e) {


  };
  closeModal(id: string) {
    this.modalService.close(id);
  }
  setBound() {
    var project = map.getMapType().getProjection();
    var getPathStr = function (overlay) {
      var pathStr = "";
      var path = overlay.getPath();
      for (var i = 0; i < path.length; i++) {
        pathStr += path[i].lng + "," + path[i].lat + ",";
      }
      pathStr = pathStr.substr(0, pathStr.length - 1);
      return pathStr;
    }
    var bdary = new BMap.Boundary();
    bdary.get('山西', function (rs) {       //获取行政区域
      //map.clearOverlays();        //清除地图覆盖物       
      var count = rs.boundaries.length; //行政区域的点有多少个
      if (count === 0) {
        alert('未能获取当前输入行政区域');
        return;
      }
      var pointArray = [];
      var pathArr = [];
      for (var i = 0; i < count; i++) {
        var ply = new BMap.Polygon(rs.boundaries[i], { strokeWeight: 1, fillOpacity: 0.05, strokeColor: "#f0f" }); //建立多边形覆盖物

        (function (ply) {
          ply.addEventListener('click', function () {
            // showResult(ply);
          });
        })(ply);
        pathArr.push(getPathStr(ply));

        map.addOverlay(ply);  //添加覆盖物
        //showResult(ply);
        var path = ply.getPath();
        var result = [];
        for (var j = 0; j < path.length; j++) {
          var px = project.lngLatToPoint(path[j]);
          result.push(px.x + ',' + px.y);
        }
        pointArray = pointArray.concat(path);
      }

      map.setViewport(pointArray);    //调整视野       
      map.setMinZoom(8);
    });
  }
  initDrawingTool():any {

    var styleOptions = {
      strokeColor: "red",    //边线颜色。
      fillColor: "red",      //填充颜色。当参数为空时，圆形将没有填充效果。
      strokeWeight: 3,       //边线的宽度，以像素为单位。
      strokeOpacity: 0.8,    //边线透明度，取值范围0 - 1。
      fillOpacity: 0.6,      //填充的透明度，取值范围0 - 1。
      strokeStyle: 'solid' //边线的样式，solid或dashed。
    }



    var mapOptions = {
      enableMapClick: true
    }


    //实例化鼠标绘制工具
    var drawingManager = new BMapLib.DrawingManager(map, {
      isOpen: true, //是否开启绘制模式
      enableDrawingTool: true, //是否显示工具栏
      drawingToolOptions: {
        // anchor: BMAP_ANCHOR_TOP_RIGHT, //位置
        offset: new BMap.Size(5, 5), //偏离值
        scale: 0.8 //工具栏缩放比例
      },
      circleOptions: styleOptions, //圆的样式
      polylineOptions: styleOptions, //线的样式
      polygonOptions: styleOptions, //多边形的样式
      rectangleOptions: styleOptions //矩形的样式
    });
    return drawingManager;
  }
}


